/*  START OF ENHANCE.C  */

/*
 * This file comprises a set of functions for enhancing the look and feel of
 * your BBS. This is not a modification per se, but rather a toolbox of
 * utilitarian functions that you can apply when and where you see fit, to
 * more customize the look and feel of your BBS.
 *
 * There are 32 functions here, including those necessary for making use
 * of ESM (any version, even 2.00 if you have registered it), as well as
 * the functions that were once included with ANSI.C, and a fair number
 * of additional functions as well.
 *
 * Each function is described and an example of how to use it is given. To
 * use these anywhere in your code, you will need to add ENHANCE.C to your
 * project file (if you use the IDE) or to your makefile (if you compile
 * on the commandline). I suggest not overlaying these functions unless you
 * have installed the XMSEMS modification which allows you to store your
 * overlays in extended or expanded memory.
 *
 * This source file has been tested with WWIV 4.20. Though it is possible
 * to make it work with prior releases, I leave that up to you to do if
 * that is your intention.
 *
 * Comments and suchlike can be sent to WWIVnet 1@3456.
 *
 * Tolkien
 * 1 @3456
 * Dec 14, 1991
 *
 */

/**/

#include "vars.h"

#pragma hdrstop

/**/

/*
 * This is a modified bitmapped sysstatus, which goes in VARDEC.H, to make
 * use of the fancy ANSI toggle (or whatever you want). Note that it is
 * padded with "fillers" in case Wayne decides to use some of those other
 * bits.
 *
 * #define sysstatus_ansi 0x0001
 * #define sysstatus_color 0x0002
 * #define sysstatus_music 0x0004
 * #define sysstatus_pause_on_page 0x0008
 * #define sysstatus_expert 0x0010
 * #define sysstatus_smw 0x0020
 * #define sysstatus_full_screen 0x0040
 * #define sysstatus_nscan_file_system 0x0080
 * #define sysstatus_funky_colors 0x0100
 * #define sysstatus_clr_scrn 0x0200
 * #define sysstatus_one 0x0400
 * #define sysstatus_two 0x0800
 * #define sysstatus_three 0x1000
 * #define sysstatus_four 0x2000
 * #define sysstatus_five 0x4000
 * #define sysstatus_fancy 0x8000
 *
 */

/**/

/*
 * You should add the following hunk of code (minus the commenting) to
 * FCNS.H. Note that some of these functions were once in ANSI.C and some
 * were in STRINGS.C (or ESM.C), so you may have some rearranging to do.
 *
 */

/* File: ENHANCE.C */

/*
* void localprt(int fc, int bc, unsigned char *s);
* int attrib(int fc, int bc);
* void writeat(int x, int y, int color, char *s);
* void writebetween(int x1, int x2, int y, int color, char *s);
* void yesorno();
* void backprint(unsigned char *s, int d1, int d2, int d3);
* void shimmer(unsigned char *s, int i);
* void slowtype(int color, unsigned char *s);
* void spin(unsigned char *s, int d1);
* void padleftwrite(int color, unsigned char *s, int endpos);
* void pla2(unsigned char *s, int *abort);
* unsigned char *get_word(char *filename);
* void prt2(int x, int i, unsigned char *s);
* unsigned char *proper(unsigned char *s);
* void center(int a, unsigned char *s);
* char *get_string(int whichstring);
* char *getstring(char *dataname, int whichstring);
* int writestring(char *datafile, int whichstring, char *writethis);
* int okfancy();
* int primetime();
* void noise(int startfreq, int endfreq, int step, int paws, int repeats);
* void send_ssm();
* void goxy(int i, int j);
* void left(int i);
* void right(int i);
* void up(int i);
* void down(int i);
* void clearline();
* void frame(int bordercolor, int x1, int y1, int x2, int y2);
* void makewin(int bordercolor, int wincolor, int x1, int y1, int x2, int y2);
* void clearscreen();
* void cleareol();
*
*/

/**/

/*
 * Here are some variables you should add to VARS.H (in both places, the
 * "normal" place at the top and the "extern" section at the bottom of
 * VARS.H).
 *
 * int lastrandom;
 *
 */

/**/

/*
 * Here are some #defines which you will wish to alter to suit your own
 * specific system configuration and preferences.
 *
 */

#define RAMDRIVE "D:\\"                /* Edit this so it is correct!      */
#define DATADIR  "C:\\WWIV\\DATA\\"    /* Edit this so it is correct!      */
#define PRIMETIMESTART 17*3600.0       /* When you want primetime to start */
#define PRIMETIMEEND 23*3600.0         /* When you want primetime to end   */

/**/

/*
 * This is the data structure for the string datafiles. Very simple.
 */

typedef struct {
  unsigned char thestring[161];
} stringrec;

/**/

void localprt(int fc, int bc, unsigned char *s)
/*
 * This function prints to the local screen only, using the attribute
 * defined as the first parameter as the color. Restores the previous
 * local screen attribute when it's done printing. Need to have <conio.h>
 * #included (in VARS.H preferably).
 *
 */
{
  unsigned char oatr;

  oatr=curatr;
  curatr=attrib(fc,bc);
  outs(s);
  curatr=oatr;
}
/*
 * Example:
 *
 * localprt(YELLOW,BLUE,"This is yellow on blue, local screen only.");
 *
 */

/**/

int attrib(int fc, int bc)
/*
 * This function takes a foreground color value and a background color value
 * and returns a single attribute byte. This is used in the localprt()
 * function above. You should have <conio.h> #included (preferably in
 * VARS.H) to make use of the constant strings as color values (such as
 * RED and YELLOW, and LIGHTBLUE, etc).
 *
 */
{
  int i;

  i = (int) (bc*16) + (fc);
  return(i);
}
/*
 * Example:
 *
 * curatr=attrib(YELLOW,MAGENTA);
 *
 */

/**/

void writeat(int x, int y, int color, char *s)
/*
 * This function will write a specified string at a specified screen location
 * using a specified color, if the user has ANSI toggled "on".
 *
 */
{
  if (okansi()) {
    goxy(x,y);
    ansic(color);
    outstr(s);
  }
}
/*
 * Example:
 *
 * writeat(1,25,4,"This is the normal color #4 on the botom screen line.");
 *
 */

/**/

void writebetween(int x1, int x2, int y, int color, char *s)
/*
 * This function will write a specified string to the screen, centered
 * between two x coordinates, in a specified color, and on a specified
 * line, if the user has ANSI toggled on.
 *
 */
{
  int i,z;

  if ((x2<=x1) || (x1<1) || (x2>80) || (y<1) || (y>50) || (strlen(s)<1)
    || (!okansi()))
    return;

  if (strlen(s)>(x2-x1+1))
    s[x2-x1+1]=0;

  i = (int) (((x2 - x1 + 1) - (strlen(s))) / 2) + x1;
  writeat(i,y,color,s);
}
/*
 * Example:
 *
 * writebetween(1,60,4,1,"This is on row 4, is color #1, between 1 and 60.");
 *
 */

/**/

void yesorno()
/*
 * This function prints out "Yes or No" in yellow, pauses, and backspaces.
 * ANSI is not required.
 */
{
  backprint("Yes or No",2,5,400);
}
/*
 * Example:
 *
 * yesorno();
 *
 */

/**/

void backprint(unsigned char *s, int d1, int d2, int d3)
/*
 * This function prints out a string, with a user-specifiable delay between
 * each character, and a user-definable pause after the entire string has
 * been printed, then it backspaces the string. The color is also definable.
 * The parameters are as follows:
 *
 * char *s : the string to print
 * int d1  : the color of the string
 * int d2  : delay between each character, in milliseconds
 * int d3  : delay between completion of string and backspacing
 *
 * ANSI is not required.
 *
 */
{
  int i1, i2, i3;

  i3 = 0;
  i1 = echo;
  echo = 1;
  i2=strlen(s);
  ansic(d1);
  delay(d2);
  while ((s[i3]) && (!hangup)) {
    outchr(s[i3]);
    delay(d2);
    i3++;
  }
  delay(d3);
  for (i3=0; i3 < i2; i3++) {
    backspace();
    delay(5);
  }
  echo = i1;
}
/*
 * Example:
 *
 * backprint("This is an example.",3,20,500);
 *
 */

/**/

void shimmer(unsigned char *s, int i)
/*
 * If the user has "fancy ANSI" on then the string to be printed will shimmer
 * through the number of colors defined in the userrec struct in VARDEC.H.
 * ANSI is checked, and if the user does not have ANSI and does not have
 * "fancy ANSI" enabled then the string is simply printed normally. A <CR> is
 * not done.
 *
 */
{
  int x,y;

  x=strlen(s);
  y=0;

  if ((okansi()) && (okfancy())) {
    do {
      if (y!=i) {
        prt(y,s);
        left(x);
      }
      y++;
    } while(y<(sizeof(thisuser.colors)));
  }
  prt(i,s);
  ansic(0);
}
/*
 * Example:
 *
 * shimmer("Whoa, wild!",6);
 *
 */

/**/

void slowtype(int color, unsigned char *s)
/*
 * This function will type out a string at random speed, as if there were
 * someone actually typing it. The color is the first parameter, just as
 * in the prt() function. A <CR> is not done.
 *
 * ANSI not required.
 *
 */
{
  int x, i;

  x=0;
  randomize();
  ansic(color);
  checkhangup();
  while ((s[x]) && (!hangup)) {
    outchr(s[x]);
    delay(random(120)+100);
    if (s[x]==32) {
      i=random(2);
      if (i==0)
        delay(random(100)+250);
    }
    x++;
  }
}
/*
 * Example:
 *
 * slowtype(3,"Hey, I'm not *really* here!");
 *
 */

/**/

void spin(unsigned char *s, int d1)
/*
 * This function will print out a string, making each character "spin"
 * using the / - \ | sequence. The color is definable and is the
 * second parameter, not the first. If the user does not have ANSI
 * then the string is simply printed normally.
 *
 */
{
  int i1, i3, dly;

  dly  = 30;
  i3   = 0;
  i1   = echo;
  echo = 1;

  if ((okansi()) && (okfancy())) {
    ansic(d1);
    while ((s[i3]) && (!hangup)) {
      delay(dly);
      outstr("/");
      left(1);
      delay(dly);
      outstr("-");
      left(1);
      delay(dly);
      outstr("\\");
      left(1);
      delay(dly);
      outstr("|");
      left(1);
      delay(dly);
      outchr(s[i3]);
      i3++;
    }
  } else {
    outstr(s);
  }
  echo = i1;
}
/*
 * Example:
 *
 * spin("This will spin and be yellow (defualt #2 color).",2);
 *
 */

/**/

void padleftwrite(int color, unsigned char *s, int endpos)
/*
 * This function prints a string and pads it with enough spaces on the
 * right of the string to make the next string print starting right after
 * endpos.  Useful for lining things up in columns, etc. The color is
 * definable and is the first parameter.
 *
 * ANSI not required.
 *
 */
{
  int i;

  prt(color,s);
  for (i=wherex();i<endpos;i++) {
    outstr(" ");
  }
}
/*
 * Example:
 *
 * padleftwrite(3,"The next string after this will be in column 41.",40);
 *
 */

/**/

void pla2(unsigned char *s, int *abort)
/*
 * This function is identical to void pla() except that it is not terminated
 * with a <CR>, so that multiple colors can be used on the same line.
 *
 */
{
  int i,next;

  i=0;
  checkhangup();
  if (hangup)
    *abort=1;
  checka(abort,&next);
  while ((s[i]) && (!(*abort))) {
    outchr(s[i++]);
    checka(abort,&next);
  }
}
/*
 * Example:
 *
 * pla2("This is on one line.",&abort);
 * pla2("Hey, this is on the same line!",&abort);
 *
 */

/**/

unsigned char *get_word(char *filename)
/*
 * This function will read a random word from a textfile and print it to the
 * screen. This function takes one parameter, the filename (in the gfiles dir)
 * from which to snag a random word/phrase.
 *
 */
{
  int f,i,ran,count;
  long p,l;
  static char s[81];

  sprintf(s,"%s%s",syscfg.gfilesdir,filename);
  if (exist(s)) {
        f=open(s,O_RDWR | O_BINARY);
    lseek(f,0L,SEEK_SET);
    l=filelength(f);
    p=0;
    count=0;
        while (p<l) {
      lseek(f,p,SEEK_SET);
      read(f,(void *)s,80);
      i=0;
      while ((i<80) && (s[i])) {
        if (s[i]==13) {
          s[i]=0;
          ++count;
        } else
          ++i;
      }
      s[80]=0;
      p += (long) (i+2);
      if (s[i-1]==1)
        s[i-1]=0;
    }
    if (lastrandom==0)
      lastrandom=1;
    do {
      ran=random(count)+1;
    } while (ran==lastrandom);
    count=0;
    lastrandom=ran;
    lseek(f,0L,SEEK_SET);
    l=filelength(f);
    p=0;
    while ((p<l) && (count<ran)) {
      lseek(f,p,SEEK_SET);
      read(f,(void *)s,80);
      i=0;
      while ((i<80) && (s[i])) {
        if (s[i]==13) {
          s[i]=0;
          ++count;
        } else
          ++i;
      }
      s[80]=0;
      p += (long) (i+2);
      if (s[i-1]==1)
        s[i-1]=0;
    }
    close(f);
    return(s);
  } else
    return("");
}
/*
 * Example:
 *
 * pl(get_word("NOUN.TXT"));
 *
 */

/**/

void prt2(int x, int i, unsigned char *s)
/*
 * This function is the same as void prt() except that it takes an extra
 * parameter, the first one, an int, which is how many spaces to put before
 * the string to be printed.
 *
 */
{
  int z;

  for (z=0; z<x; z++)
        outstr(" ");
  ansic(i);
  outstr(s);
  ansic(0);
}
/*
 * Example:
 *
 * prt2(30,1,"Much more memory effective than...");
 * prt(1,"                               this.");
 *
 */

/**/

unsigned char *proper(unsigned char *s)
/*
 * This function will return a "properized" replica of the string it is
 * fed. In other words, "jon doe" would become "Jon Doe".
 *
 */
{
  int i,f;
  static char o[81];
  char z[81];

  strcpy(z,s);
  strcpy(z,strupr(z));
  f=1;
  for (i=0; i<strlen(z); i++) {
    if (f) {
      if ((z[i]>='A') && (z[i]<='Z'))
        f=0;
      o[i]=z[i];
    } else {
      if ((z[i]>='A') && (z[i]<='Z'))
        o[i]=z[i]-'A'+'a';
      else {
        if ((z[i]>=' ') && (z[i]<='/'))
          f=1;
        o[i]=z[i];
      }
    }
  }
  o[i++]=0;
  return(o);
}
/*
 * Example:
 *
 * outstr(proper(thisuser.realname));
 *
 */

/**/

void center(int a, unsigned char *s)
/*
 * This function centers a string on a line, and takes into account the
 * embedded WWIV color codes (ASCII 003's).
 *
 */
{
  char s1[160];
  int i,x,col,lenx;
  div_t len;

  i = 0;
  x = 0;
  lenx = strlen(s);

  while (s[x]) {
    if (s[x]==3) {
      lenx--;
      lenx--;
    }
    x++;
  }
  if (thisuser.screenchars > lenx) {
    col = thisuser.screenchars - lenx;
    len = div(col,2);
    i = len.quot;
  }
  strcpy(s1,"");
  while (i>0) {
    strcat(s1," ");
    i--;
  }
  strcat(s1,s);
  ansic(a);
  outstr(s1);
}
/*
 * Example:
 *
 * center(6,"Hey Jude, don't make it bad.");
 *
 */

/**/

/*
 * This function is the one that retrieves the string from STRINGS.DAT
 * If there is an error reading the file the BBS will abort (much like
 * it does when it cannot find CONFIG.DAT, etc). It will try first for
 * the ramdrive. If it cannot access that file then it will look in
 * the data directory. *Make sure* you specify the directories right.
 * The datafile filename should be the first parameter in this case,
 * followed by the record number to retrieve.
 */

char *get_string(int whichstring)
{
  static char s[161];
  char s1[81];
  int i;
  stringrec astring;

  sprintf(s1,"%sSTRINGS.DAT",RAMDRIVE);
  i=open(s1,O_RDWR | O_BINARY);
  /* The above line is the path to the ramdrive and the STRINGS.DAT file.
   * If you don't use a ramdrive then make this the path to your data
   * directory.
   */
  if (i<0) {
    sprintf(s1,"%sSTRINGS.DAT",DATADIR);
    i=open(s1,O_RDWR | O_BINARY);
    /* The above line should be the path of your BBS data directory which
     * should be the permanent residence of the STRINGS.DAT file. ESM.EXE
     * should either be in the WWIV main dir or in the path or, preferably,
     * in the WWIV main dir and have that in the path.
     */
    if (i<0) {
      printf("\r\n\7Error reading STRINGS.DAT.\7\r\n");
      end_bbs(noklevel);
    }
  }
  lseek(i,((long)(whichstring-1))*(sizeof(astring)),SEEK_SET);
  read(i,(void *)&astring,sizeof(astring));
  sprintf(s,"%s",&astring.thestring);
  close(i);
  return(s);
}
/*
 * Example:
 *
 * outstr(get_string(3));
 *
 */

/**/

/*
 * The following is a getstring function that allows you to specify
 * the string datafile filename from which to retrieve the string.
 * This would allow you to have separate string datafiles for each
 * WWIV module. For example, a BBS.DAT, a BBSUTL.DAT, a COM.DAT, etc.
 * Note the slightly different name for this function to distinguish
 * it from the other. The extension .DAT is automatically used,
 * so >do not< use it in your getstring calls.
 */

char *getstring(char *dataname, int whichstring)
{
  static char s[161];
  char s1[81];
  int i;
  stringrec astring;

  sprintf(s1,"%s%s.DAT",RAMDRIVE,dataname);
  i=open(s1,O_RDWR | O_BINARY);
  if (i<0) {
    sprintf(s1,"%s%s.DAT",DATADIR,dataname);
    i=open(s1,O_RDWR | O_BINARY);
    if (i<0) {
      sprintf(s1,"\r\n\7Error reading %s.DAT.\7\r\n",dataname);
      printf(s1);
      end_bbs(noklevel);
    }
  }
  lseek(i,((long)(whichstring-1))*(sizeof(astring)),SEEK_SET);
  read(i,(void *)&astring,sizeof(astring));
  sprintf(s,"%s",&astring.thestring);
  close(i);
  return(s);
}
/*
 * Example:
 *
 * outstr(getstring("RANDOM",5));
 *
 */

/**/

/*
 * The next function will write a specified string at a specified location
 * in a specified string datafile. If the write fails this function will
 * not abort the BBS but will retuen a 0. It returns a 1 if the write is
 * successful.
 *
 */

int writestring(char *datafile, int whichstring, char *writethis)
{
  char s[81],s1[161];
  stringrec astring;
  int i;
  long l;

  sprintf(s,"%s%s.DAT",RAMDRIVE,datafile);
  i=open(s,O_RDWR | O_BINARY);
  if (i<0) {
    sprintf(s,"%s%s.DAT",DATADIR,datafile);
    i=open(s,O_RDWR | O_BINARY);
    if (i<0) {
      return(0);
    }
  }
  lseek(i,((long)(whichstring-1))*(sizeof(astring)),SEEK_SET);
  l=strlen(writethis);
  strcpy(s1,writethis);
  if (write(i, (void *)s1, l)!=-1) {
    close(i);
    return(1);
  } else
    return(0);
}
/*
 * Example:
 *
 * if (!writestring("RANDOM","Stash this string in the third record.",3))
 *   outstr("String write failed. RANDOM.DAT isn't there?");
 *
 */

/**/

int okfancy()
/*
 * Checks to see if the sysstatus_fancy bit is toggled on. Use it as you
 * wish. I use it for determining how much neat ANSI to send at people,
 * since some like it and some do not.
 *
 */
{
  if (thisuser.sysstatus & sysstatus_fancy)
    return(1);
  else
    return(0);
}
/*
 * Example:
 *
 * if (okfancy())
 *   ansic(6);
 * pl(s);
 *
 */

/**/

int primetime()
/*
 * This function checks to see if the current time is between the declared
 * prime time, for uploading, downloading, bimodeming, etc. If it is then
 * this function is returned as true (1), else it is false (0);
 *
 */
{
  if (!(thisuser.exempt & exempt_primetime)) {
    if ((timer()>=PRIMETIMESTART) && (timer()<=PRIMETIMEEND))
      return(1);
    else
      return(0);
  } else
    return(0);
}
/*
 * Example:
 *
 * if (primetime())
 *   pl("Sorry, it's primetime. Xfers are closed. Call back later.");
 *
 */

/**/

void noise(int startfreq, int endfreq, int step, int paws, int repeats)
/*
 * This function takes several parameters. Depending on these parameters,
 * a noise will be made locally. We use this for such things as newuser
 * logon, file access, chain access, logoff, chat, etc.
 *
 * The parameters are as follows:
 *
 * startfreq : The start frequency in hertz.
 * endfreq   : The end frequency in hertz.
 * step      : The number of hertz to step between startfreq and endfreq.
 *             This number should be negative if endfreq is less than
                                                         startfreq.
 * paws      : The paws in milliseconds between repetitions.
 * repeats   : The number of times to repeat the entire sequence.
 *
 * ANSI not required.
 *
 */
{
  int i,x,z;

  x=0;
  if (startfreq<endfreq) {
    do {
      z=startfreq;
      do {
        sound(z);
        delay(paws);
        z+=step;
      } while ((z <= endfreq) && (!hangup));
      x++;
    } while((x < repeats) && (!hangup));
  } else {
    do {
      z=startfreq;
      do {
        sound(z);
        delay(paws);
        z+=step;
      } while ((z >= endfreq) && (!hangup));
      x++;
    } while((x < repeats) && (!hangup));
  }
  nosound();
}
/*
 * Example:
 *
 * noise(100,800,25,20,3);
 * newuser();
 *
 */

/**/

void send_ssm()
/*
 * This function allows a caller, if his/her security level is higher than
 * that of a newuser, to send a one-line message to another caller, that will
 * be seen the next time that caller logs on. It is just like the messages
 * such as "Mr. Dude read your mail on XX/XX/XX.", and shown in the same
 * place. (SSM.EXE included in SSM###.ZIP, where ### is the current version
 * number, will also allow this function, and more.)
 *
 */
{
  char s1[81],*ss;
  int un;

  if (actsl > syscfg.newusersl) {
    clearscreen();
    outstr("Send a short message? ");
    if (yn()) {
      nl();
      prt(1,"Enter caller's name or number: ");
      ansic(0);
      input(s1,30);
      ss=strchr(s1,64);
      if (ss==NULL) {
        un=finduser1(s1);
        if (un>0) {
          nl();
          pl("Enter short message now.");
          prt(2,": ");
          mpl(70);
          ansic(4);
          inputl(s1,70);
          ssm(un,0,s1);
          nl();
          pl("Short message has been sent!");
          nl();
        } else {
          pl("Who?!?");
          nl();
        }
      } else {
        pl("Can't send a short message across the net!");
        nl();
      }
    }
  }
}

/**/

void goxy(int i, int j)
/*
 * This function will position the cursor at x:i, y:j coordinates on the
 * screen. If the user has no ANSI then nothing happens.
 *
 */
{
  char s[12];

  if (okansi()) {
    sprintf(s,"\x1b[%d;%dH",j,i);
    outstr(s);
  }
}
/*
 * Example:
 *
 * goxy(12,22);
 *
 */

/**/

void left(int i)
/*
 * This function will reposition the cursor i spaces to the left, or if the
 * cursor is on the left side of the screen already then it will not move.
 * If the user has no ANSI then nothing happens.
 *
 */
{
  char s[10];

  if (okansi()) {
    sprintf(s,"\x1b[%dD",i);
    outstr(s);
  }
}
/*
 * Example:
 *
 * left(6);
 *
 */

/**/

void right(int i)
/*
 * This function repositions the cursor i spaces to the right, not moving it
 * at all if it is already on the righthand side of the screen. If the
 * user hasn't got ANSI then nothing happens.
 *
 */
{
  char s[10];

  if (okansi()) {
    sprintf(s,"\x1b[%dC",i);
    outstr(s);
  }
}
/*
 * Example:
 *
 * right(12);
 *
 */

/**/

void up(int i)
/*
 * This function repositions the cursor i spaces upwards, not moving it at
 * all if it is already on the upper line of the screen. If the user has no
 * ANSI then nothing happens.
 *
 */
{
  char s[10];

  if (okansi()) {
    sprintf(s,"\x1b[%dA",i);
    outstr(s);
  }
}
/*
 * Example:
 *
 * up(25);
 *
 */

/**/

void down(int i)
/*
 * This function repositions the cursor i spaces to the bottom, not moving it
 * at all if it is already on the bottom line of the screen. If the user has
 * no ANSI then nothing happens.
 *
 */
{
  char s[10];

  if (okansi()) {
    sprintf(s,"\x1b[%dB",i);
    outstr(s);
  }
}
/*
 * Example:
 *
 * down(1);
 *
 */

/**/

void clearline()
/*
 * This function clears the current line, if the user has ANSI, otherwise
 * nothing happens.
 *
 */
{
  if (okansi()) {
    outstr("\x1b[K");
  }
}
/*
 * Example:
 *
 * clearline();
 *
 */

/**/

void frame(int bordercolor, int x1, int y1, int x2, int y2)
/*
 * This function creates a colored frame around an area on the screen. The
 * screen within the frame is not cleared or overwritten. Nice for special
 * titling effects and whatnot.
 *
 */
{
  char s[91];
  int i,z;

  if (okansi()) {
    ansic(bordercolor);
    strcpy(s,"");
    for (i=0;i<(x2-x1-1);i++)
      strcat(s,"");
    strcat(s,"");
    goxy(x1,y1);
    outstr(s);
    if ((y2-y1)>1) {
      for (z=0;z<(y2-y1-1);z++) {
        goxy(x1,y1+1+z);
        outstr("");
        goxy(x2,y1+1+z);
        outstr("");
      }
    }
    strcpy(s,"");
    for (i=0;i<(x2-x1-1);i++)
      strcat(s,"");
    strcat(s,"");
    goxy(x1,y2);
    outstr(s);
  }
}
/*
 * Example:
 *
 * frame(4,11,3,70,5);
 * writebetween(12,69,4,3,"This is color #3 centered in the frame.");
 *
 */

/**/

void makewin(int bordercolor, int wincolor, int x1, int y1, int x2, int y2)
/*
 * This function will create an ansi box with (x1,y1) the top left corner
 * and (x2,y2) the bottom right corner of the box in the color specified.
 * The inside of the box is cleared and set to wincolor.
 *
 */
{
  char s[91];
  int i,z;

  if (okansi()) {
    frame(bordercolor,x1,y1,x2,y2);
    ansic(wincolor);
    if ((y2-y1)>1) {
      for (z=0;z<(y2-y1-1);z++) {
        strcpy(s," ");
        for (i=0;i<(x2-x1-2);i++)
          strcat(s," ");
        goxy(x1+1,y1+1+z);
        outstr(s);
      }
    }
  }
}
/*
 * Example:
 *
 * makewin(4,2,11,3,70,23);
 *
 */

/**/

void clearscreen()
/*
 * This function clears the screen, using ANSI escape codes if the user
 * has ANSI toggled on, else using the normal WWIV method of spitting out
 * an ASCII 012.
 *
 */
{
  if (okansi()) {
    outchr(12);
    outstr("\x1b[2J");
  } else {
    outchr(12);
  }
}
/*
 * Example:
 *
 * clearscreen();
 *
 */

/**/

void cleareol()
/*
 * If the user has ANSI then this function will clear from the current cursor
 * position to the end of the line.
 *
 */
{
  if (okansi())
    outstr("\x1b[K");
}
/*
 * Example:
 *
 * cleareol();
 *
 */

/*  END OF ENHANCE.C  */